@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,18 +21,26 @@
 
 #include "Configuration.h"
 
+#include "../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 #include "DicomWebServers.h"
 
-#include <Plugins/Samples/Common/OrthancPluginCppWrapper.h>
-#include <Core/Toolbox.h>
+#include <Compatibility.h>
+#include <Logging.h>
+#include <Toolbox.h>
 
 #include <fstream>
-#include <json/reader.h>
 #include <boost/regex.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
 
+// Assume Latin-1 encoding by default (as in the Orthanc core)
+static Orthanc::Encoding defaultEncoding_ = Orthanc::Encoding_Latin1;
+static std::unique_ptr<OrthancPlugins::OrthancConfiguration> configuration_;
+static bool serversInDatabase_ = false;
+static const int32_t GLOBAL_PROPERTY_SERVERS = 5468;
+
+
 namespace OrthancPlugins
 {
   bool LookupHttpHeader(std::string& value,
@@ -140,7 +148,7 @@ namespace OrthancPlugins
   {
     OrthancPluginDictionaryEntry entry;
     
-    if (OrthancPluginLookupDictionary(OrthancPlugins::GetGlobalContext(), &entry, name.c_str()) == OrthancPluginErrorCode_Success)
+    if (OrthancPluginLookupDictionary(GetGlobalContext(), &entry, name.c_str()) == OrthancPluginErrorCode_Success)
     {
       target = Orthanc::DicomTag(entry.group, entry.element);
       return true;
@@ -155,9 +163,7 @@ namespace OrthancPlugins
   void ParseJsonBody(Json::Value& target,
                      const OrthancPluginHttpRequest* request)
   {
-    Json::Reader reader;
-    if (!reader.parse(reinterpret_cast<const char*>(request->body),
-                      reinterpret_cast<const char*>(request->body) + request->bodySize, target))
+    if (!OrthancPlugins::ReadJson(target, request->body, request->bodySize))
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat,
                                       "A JSON file was expected");
@@ -286,16 +292,11 @@ namespace OrthancPlugins
 
   namespace Configuration
   {
-    // Assume Latin-1 encoding by default (as in the Orthanc core)
-    static Orthanc::Encoding defaultEncoding_ = Orthanc::Encoding_Latin1;
-    static std::auto_ptr<OrthancConfiguration> configuration_;
-
-
     void Initialize()
     {
       configuration_.reset(new OrthancConfiguration);
       
-      OrthancPlugins::OrthancConfiguration global;
+      OrthancConfiguration global;
       global.GetSection(*configuration_, "DicomWeb");
 
       std::string s;
@@ -304,10 +305,22 @@ namespace OrthancPlugins
         defaultEncoding_ = Orthanc::StringToEncoding(s.c_str());
       }
 
-      OrthancPlugins::OrthancConfiguration servers;
-      configuration_->GetSection(servers, "Servers");
-      OrthancPlugins::DicomWebServers::GetInstance().Load(servers.GetJson());
+      if (!configuration_->LookupBooleanValue(serversInDatabase_, "ServersInDatabase"))
+      {
+        serversInDatabase_ = false;
+      }
 
+      if (serversInDatabase_)
+      {
+        LOG(INFO) << "The DICOMweb plugin stores the DICOMweb servers in the Orthanc database";
+      }
+      else
+      {
+        LOG(INFO) << "The DICOMweb plugin reads the DICOMweb servers from the configuration file";
+      }
+
+      DicomWebServers::GetInstance().Clear();
+        
       // Check configuration during initialization
       GetMetadataMode(Orthanc::ResourceType_Study);
       GetMetadataMode(Orthanc::ResourceType_Series);
@@ -373,7 +386,7 @@ namespace OrthancPlugins
     
     std::string GetOrthancApiRoot()
     {
-      std::string root = OrthancPlugins::Configuration::GetDicomWebRoot();
+      std::string root = Configuration::GetDicomWebRoot();
       std::vector<std::string> tokens;
       Orthanc::Toolbox::TokenizeString(tokens, root, '/');
 
@@ -446,10 +459,10 @@ namespace OrthancPlugins
 
 
     static bool LookupHttpHeader2(std::string& value,
-                                  const OrthancPlugins::HttpClient::HttpHeaders& headers,
+                                  const HttpClient::HttpHeaders& headers,
                                   const std::string& name)
     {
-      for (OrthancPlugins::HttpClient::HttpHeaders::const_iterator
+      for (HttpClient::HttpHeaders::const_iterator
              it = headers.begin(); it != headers.end(); ++it)
       {
         if (boost::iequals(it->first, name))
@@ -463,7 +476,7 @@ namespace OrthancPlugins
     }
 
 
-    std::string GetBaseUrl(const OrthancPlugins::HttpClient::HttpHeaders& headers)
+    std::string GetBaseUrl(const HttpClient::HttpHeaders& headers)
     {
       assert(configuration_.get() != NULL);
       std::string host = configuration_->GetStringValue("Host", "");
@@ -523,7 +536,7 @@ namespace OrthancPlugins
 
     std::string GetBaseUrl(const OrthancPluginHttpRequest* request)
     {
-      OrthancPlugins::HttpClient::HttpHeaders headers;
+      HttpClient::HttpHeaders headers;
 
       std::string value;
       if (LookupHttpHeader(value, request, "forwarded"))
@@ -586,8 +599,8 @@ namespace OrthancPlugins
       }
       else
       {
-        OrthancPlugins::LogError("Unsupported return MIME type: " + accept +
-                                 ", will return DICOM+JSON");
+        LogError("Unsupported return MIME type: " + accept +
+                 ", will return DICOM+JSON");
         return false;
       }
     }
@@ -612,7 +625,7 @@ namespace OrthancPlugins
     {
       std::string accept;
 
-      if (OrthancPlugins::LookupHttpHeader(accept, request, "accept"))
+      if (LookupHttpHeader(accept, request, "accept"))
       {
         return IsXmlExpected(accept);
       }
@@ -711,5 +724,58 @@ namespace OrthancPlugins
           throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
       }
     }
+
+
+    void LoadDicomWebServers()
+    {
+      if (serversInDatabase_)
+      {
+        // New in DICOMweb 1.5
+        OrthancString property;
+        property.Assign(OrthancPluginGetGlobalProperty(
+                          GetGlobalContext(), GLOBAL_PROPERTY_SERVERS, "{}"));
+
+        if (property.GetContent() == NULL)
+        {
+          DicomWebServers::GetInstance().Clear();
+        }
+        else
+        {
+          try
+          {
+            DicomWebServers::GetInstance().UnserializeGlobalProperty(property.GetContent());
+          }
+          catch (Orthanc::OrthancException&)
+          {
+            DicomWebServers::GetInstance().Clear();
+            LOG(ERROR) << "Cannot read the DICOMweb servers from the database, no server will be defined";
+          }
+        }
+      }
+      else
+      {
+        OrthancConfiguration servers;
+        configuration_->GetSection(servers, "Servers");
+        DicomWebServers::GetInstance().LoadGlobalConfiguration(servers.GetJson());
+      }
+    }
+
+    
+    void SaveDicomWebServers()
+    {
+      if (serversInDatabase_)
+      {
+        // New in DICOMweb 1.5
+        std::string property;
+        DicomWebServers::GetInstance().SerializeGlobalProperty(property);
+
+        if (OrthancPluginSetGlobalProperty(
+              OrthancPlugins::GetGlobalContext(), GLOBAL_PROPERTY_SERVERS, property.c_str()) !=
+            OrthancPluginErrorCode_Success)
+        {
+          LOG(ERROR) << "Cannot write the DICOMweb servers into the database";
+        }
+      }
+    }
   }
 }
