@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,8 +21,8 @@
 
 #pragma once
 
-#include <Core/DicomFormat/DicomTag.h>
-#include <Core/Enumerations.h>
+#include <DicomFormat/DicomTag.h>
+#include <Enumerations.h>
 
 #include <orthanc/OrthancCPlugin.h>
 #include <json/value.h>
@@ -133,5 +133,9 @@ namespace OrthancPlugins
 
     void GetExtrapolatedMetadataTags(std::set<Orthanc::DicomTag>& tags,
                                      Orthanc::ResourceType level);
+
+    void LoadDicomWebServers();
+
+    void SaveDicomWebServers();
   }
 }
