@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -18,6 +18,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  **/
 
+#include "../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 #include "DicomWebClient.h"
 #include "DicomWebServers.h"
 #include "QidoRs.h"
@@ -25,9 +26,9 @@
 #include "WadoRs.h"
 #include "WadoUri.h"
 
-#include <Plugins/Samples/Common/OrthancPluginCppWrapper.h>
-#include <Core/SystemToolbox.h>
-#include <Core/Toolbox.h>
+#include <Logging.h>
+#include <SystemToolbox.h>
+#include <Toolbox.h>
 
 #include <EmbeddedResources.h>
 
@@ -127,6 +128,8 @@ void ListServerOperations(OrthancPluginRestOutput* output,
     case OrthancPluginHttpMethod_Delete:
     {
       OrthancPlugins::DicomWebServers::GetInstance().DeleteServer(request->groups[0]);
+      OrthancPlugins::Configuration::SaveDicomWebServers();
+
       std::string answer = "{}";
       OrthancPluginAnswerBuffer(context, output, answer.c_str(), answer.size(), "application/json");
       break;
@@ -140,6 +143,8 @@ void ListServerOperations(OrthancPluginRestOutput* output,
       Orthanc::WebServiceParameters parameters(body);
       
       OrthancPlugins::DicomWebServers::GetInstance().SetServer(request->groups[0], parameters);
+      OrthancPlugins::Configuration::SaveDicomWebServers();
+      
       std::string answer = "{}";
       OrthancPluginAnswerBuffer(context, output, answer.c_str(), answer.size(), "application/json");
       break;
@@ -448,6 +453,36 @@ void ServeDicomWebClient(OrthancPluginRestOutput* output,
 #endif
 
 
+static OrthancPluginErrorCode OnChangeCallback(OrthancPluginChangeType changeType, 
+                                               OrthancPluginResourceType resourceType, 
+                                               const char *resourceId)
+{
+  try
+  {
+    switch (changeType)
+    {
+      case OrthancPluginChangeType_OrthancStarted:
+        OrthancPlugins::Configuration::LoadDicomWebServers();
+        break;
+
+      default:
+        break;
+    }
+  }
+  catch (Orthanc::OrthancException& e)
+  {
+    LOG(ERROR) << "Exception: " << e.What();
+  }
+  catch (...)
+  {
+    LOG(ERROR) << "Uncatched native exception";
+  }  
+
+  return OrthancPluginErrorCode_Success;
+}
+
+
+
 extern "C"
 {
   ORTHANC_PLUGINS_API int32_t OrthancPluginInitialize(OrthancPluginContext* context)
@@ -455,7 +490,14 @@ extern "C"
     assert(DisplayPerformanceWarning(context));
 
     OrthancPlugins::SetGlobalContext(context);
+    
+#if ORTHANC_FRAMEWORK_VERSION_IS_ABOVE(1, 7, 2)
+    Orthanc::Logging::InitializePluginContext(context);
+#else
     Orthanc::Logging::Initialize(context);
+#endif
+
+    Orthanc::Logging::EnableInfoLevel(true);
 
     /* Check the version of the Orthanc core */
     if (OrthancPluginCheckVersion(context) == 0)
@@ -532,6 +574,8 @@ extern "C"
         OrthancPlugins::RegisterRestCallback<RetrieveInstanceRendered>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/rendered", true);
         OrthancPlugins::RegisterRestCallback<RetrieveFrameRendered>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames/([^/]*)/rendered", true);
 
+        OrthancPluginRegisterOnChangeCallback(context, OnChangeCallback);
+
 
         // Extend the default Orthanc Explorer with custom JavaScript for STOW client
         std::string explorer;
