@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,13 +21,14 @@
 
 #include "QidoRs.h"
 
+#include "../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 #include "Configuration.h"
 #include "DicomWebFormatter.h"
 
-#include <Core/DicomFormat/DicomMap.h>
-#include <Core/DicomFormat/DicomTag.h>
-#include <Core/Toolbox.h>
-#include <Plugins/Samples/Common/OrthancPluginCppWrapper.h>
+#include <DicomFormat/DicomMap.h>
+#include <DicomFormat/DicomTag.h>
+#include <Logging.h>
+#include <Toolbox.h>
 
 #include <list>
 #include <boost/lexical_cast.hpp>
@@ -169,10 +170,10 @@ namespace
             std::vector<std::string> tags;
             Orthanc::Toolbox::TokenizeString(tags, value, ',');
             
-            for (size_t i = 0; i < tags.size(); i++)
+            for (size_t j = 0; j < tags.size(); j++)
             {
               Orthanc::DicomTag tag(0, 0);
-              if (OrthancPlugins::ParseTag(tag, tags[i]))
+              if (OrthancPlugins::ParseTag(tag, tags[j]))
               {
                 includeFields_.push_back(tag);
               }
@@ -491,11 +492,7 @@ static void ApplyMatcher(OrthancPluginRestOutput* output,
   LOG(INFO) << "Body of the call from QIDO-RS to /tools/find: " << find.toStyledString();
   
   std::string body;
-
-  {
-    Json::FastWriter writer;
-    body = writer.write(find);
-  }
+  OrthancPlugins::WriteFastJson(body, find);
   
   Json::Value resources;
   if (!OrthancPlugins::RestApiPost(resources, "/tools/find", body, false) ||
@@ -525,12 +522,6 @@ static void ApplyMatcher(OrthancPluginRestOutput* output,
       Orthanc::DicomMap source;
       source.FromDicomAsJson(tags);
 
-      std::string wadoUrl = OrthancPlugins::Configuration::GetWadoUrl(
-        wadoBase,
-        source.GetStringValue(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, "", false),
-        source.GetStringValue(Orthanc::DICOM_TAG_SERIES_INSTANCE_UID, "", false),
-        source.GetStringValue(Orthanc::DICOM_TAG_SOP_INSTANCE_UID, "", false));
-
       Orthanc::DicomMap target;
       target.Assign(derivedTags);
       matcher.ExtractFields(target, source, wadoBase, level);
